@@ -28,7 +28,7 @@ couchdb_document_contact_get_first_name (CouchDBDocument *document)
 	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
 	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
 
-	return json_object_get_string_member (json_node_get_object (document->root_node), "first_name");
+	return couchdb_document_get_string_field (document, "first_name");
 }
 
 void
@@ -38,7 +38,7 @@ couchdb_document_contact_set_first_name (CouchDBDocument *document, const char *
 	g_return_if_fail (couchdb_document_is_contact (document));
 	g_return_if_fail (first_name != NULL);
 
-	json_object_set_string_member (json_node_get_object (document->root_node), "first_name", first_name);
+	couchdb_document_set_string_field (document, "first_name", first_name);
 }
 
 const char *
@@ -47,7 +47,7 @@ couchdb_document_contact_get_last_name (CouchDBDocument *document)
 	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
 	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
 
-	return json_object_get_string_member (json_node_get_object (document->root_node), "last_name");
+	return couchdb_document_get_string_field (document, "last_name");
 }
 
 void
@@ -57,7 +57,7 @@ couchdb_document_contact_set_last_name (CouchDBDocument *document, const char *l
 	g_return_if_fail (couchdb_document_is_contact (document));
 	g_return_if_fail (last_name != NULL);
 
-	json_object_set_string_member (json_node_get_object (document->root_node), "last_name", last_name);
+	couchdb_document_set_string_field (document, "last_name", last_name);
 }
 
 const char *
@@ -68,7 +68,7 @@ couchdb_document_contact_get_birth_date (CouchDBDocument *document)
 	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
 	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
 
-	return json_object_get_string_member (json_node_get_object (document->root_node), "birth_date");
+	return couchdb_document_get_string_field (document, "birth_date");
 }
 
 void
@@ -78,7 +78,7 @@ couchdb_document_contact_set_birth_date (CouchDBDocument *document, const char *
 	g_return_if_fail (couchdb_document_is_contact (document));
 	g_return_if_fail (birth_date != NULL);
 
-	json_object_set_string_member (json_node_get_object (document->root_node), "birth_date", birth_date);
+	couchdb_document_set_string_field (document, "birth_date", birth_date);
 }
 
 static void
@@ -94,6 +94,7 @@ foreach_object_cb (JsonObject *object,
 
 		sf = couchdb_struct_field_new_from_json_object (
 			json_object_ref (json_node_get_object (member_node)));
+		couchdb_struct_field_set_uuid (sf, member_name);
 		*list = g_slist_prepend (*list, sf);
 	}
 }
@@ -153,7 +154,9 @@ couchdb_document_contact_set_email_addresses (CouchDBDocument *document, GSList
 			json_object_set_string_member (this_address, "description",
 						       couchdb_document_contact_email_get_description (sf));
 
-			json_object_set_object_member (addresses_json, address_str, this_address);
+			json_object_set_object_member (addresses_json,
+						       couchdb_struct_field_get_uuid (sf),
+						       this_address);
 			/* FIXME: crashes if we _unref json_object_unref (this_address); */
 		}
 	}
@@ -206,7 +209,9 @@ couchdb_document_contact_set_phone_numbers (CouchDBDocument *document, GSList *l
 			json_object_set_int_member (this_phone, "priority",
 						    couchdb_document_contact_phone_get_priority (sf));
 
-			json_object_set_object_member (phone_numbers, number_str, this_phone);
+			json_object_set_object_member (phone_numbers,
+						       couchdb_struct_field_get_uuid (sf),
+						       this_phone);
 			/* FIXME: crashes if we _unref json_object_unref (this_phone); */
 		}
 	}
@@ -267,7 +272,9 @@ couchdb_document_contact_set_addresses (CouchDBDocument *document, GSList *list)
 			json_object_set_string_member (this_address, "description",
 						       couchdb_document_contact_address_get_description (sf));
 
-			json_object_set_object_member (addresses, street_str, this_address);
+			json_object_set_object_member (addresses,
+						       couchdb_struct_field_get_uuid (sf),
+						       this_address);
 			/* FIXME: crashes if we _unref json_object_unref (this_address); */
 		}
 	}
@@ -277,11 +284,19 @@ couchdb_document_contact_set_addresses (CouchDBDocument *document, GSList *list)
 }
 
 CouchDBStructField *
-couchdb_document_contact_email_new (const char *address, const char *description)
+couchdb_document_contact_email_new (const char *uuid, const char *address, const char *description)
 {
 	CouchDBStructField *sf;
 
 	sf = couchdb_struct_field_new_from_json_object (json_object_new ());
+	if (uuid != NULL)
+		couchdb_struct_field_set_uuid (sf, uuid);
+	else {
+		char *new_uuid = generate_uuid ();
+		couchdb_struct_field_set_uuid (sf, new_uuid);
+		g_free (new_uuid);
+	}
+
 	if (address)
 		couchdb_document_contact_email_set_address (sf, address);
 	if (description)
@@ -295,7 +310,7 @@ couchdb_document_contact_email_get_address (CouchDBStructField *sf)
 {
 	g_return_val_if_fail (sf != NULL, NULL);
 
-	return json_object_get_string_member (sf->json_object, "address");
+	return couchdb_struct_field_get_string_field (sf, "address");
 }
 
 void
@@ -304,7 +319,7 @@ couchdb_document_contact_email_set_address (CouchDBStructField *sf, const char *
 	g_return_if_fail (sf != NULL);
 	g_return_if_fail (email != NULL);
 
-	json_object_set_string_member (sf->json_object, "address", email);
+	couchdb_struct_field_set_string_field (sf, "address", email);
 }
 
 const char *
@@ -312,24 +327,31 @@ couchdb_document_contact_email_get_description (CouchDBStructField *sf)
 {
 	g_return_val_if_fail (sf != NULL, NULL);
 
-	return json_object_get_string_member (sf->json_object, "description");
+	return couchdb_struct_field_get_string_field (sf, "description");
 }
 
 void
 couchdb_document_contact_email_set_description (CouchDBStructField *sf, const char *description)
 {
 	g_return_if_fail (sf != NULL);
-	g_return_if_fail (description != NULL);
 
-	json_object_set_string_member (sf->json_object, "description", description);
+	couchdb_struct_field_set_string_field (sf, "description", description);
 }
 
 CouchDBStructField *
-couchdb_document_contact_phone_new (const char *number, const char *description, gint priority)
+couchdb_document_contact_phone_new (const char *uuid, const char *number, const char *description, gint priority)
 {
 	CouchDBStructField *sf;
 
 	sf = couchdb_struct_field_new_from_json_object (json_object_new ());
+	if (uuid != NULL)
+		couchdb_struct_field_set_uuid (sf, uuid);
+	else {
+		char *new_uuid = generate_uuid ();
+		couchdb_struct_field_set_uuid (sf, new_uuid);
+		g_free (new_uuid);
+	}
+
 	if (number)
 		couchdb_document_contact_phone_set_number (sf, number);
 	if (description)
@@ -344,7 +366,7 @@ couchdb_document_contact_phone_get_priority (CouchDBStructField *sf)
 {
 	g_return_val_if_fail (sf != NULL, 0);
 
-	return json_object_get_int_member (sf->json_object, "priority");
+	return couchdb_struct_field_get_int_field (sf, "priority");
 }
 
 void
@@ -352,15 +374,15 @@ couchdb_document_contact_phone_set_priority (CouchDBStructField *sf, gint priori
 {
 	g_return_if_fail (sf != NULL);
 
-	json_object_set_int_member (sf->json_object, "priority", priority);
+	couchdb_struct_field_set_int_field (sf, "priority", priority);
 }
 
 const char *
 couchdb_document_contact_phone_get_number (CouchDBStructField *sf)
 {
-	g_return_val_if_fail (sf->json_object, NULL);
+	g_return_val_if_fail (sf != NULL, NULL);
 
-	return json_object_get_string_member (sf->json_object, "number");
+	return couchdb_struct_field_get_string_field (sf, "number");
 }
 
 void
@@ -369,7 +391,7 @@ couchdb_document_contact_phone_set_number (CouchDBStructField *sf, const char *n
 	g_return_if_fail (sf != NULL);
 	g_return_if_fail (number != NULL);
 
-	json_object_set_string_member (sf->json_object, "number", number);
+	couchdb_struct_field_set_string_field (sf, "number", number);
 }
 
 const char *
@@ -377,20 +399,20 @@ couchdb_document_contact_phone_get_description (CouchDBStructField *sf)
 {
 	g_return_val_if_fail (sf != NULL, NULL);
 
-	return json_object_get_string_member (sf->json_object, "description");
+	return couchdb_struct_field_get_string_field (sf, "description");
 }
 
 void
 couchdb_document_contact_phone_set_description (CouchDBStructField *sf, const char *description)
 {
 	g_return_if_fail (sf != NULL);
-	g_return_if_fail (description != NULL);
 
-	json_object_set_string_member (sf->json_object, "description", description);
+	couchdb_struct_field_set_string_field (sf, "description", description);
 }
 
 CouchDBStructField *
-couchdb_document_contact_address_new (const char *street,
+couchdb_document_contact_address_new (const char *uuid,
+				      const char *street,
 				      const char *city,
 				      const char *state,
 				      const char *country,
@@ -401,6 +423,14 @@ couchdb_document_contact_address_new (const char *street,
 	CouchDBStructField *sf;
 
 	sf = couchdb_struct_field_new_from_json_object (json_object_new ());
+	if (uuid != NULL)
+		couchdb_struct_field_set_uuid (sf, uuid);
+	else {
+		char *new_uuid = generate_uuid ();
+		couchdb_struct_field_set_uuid (sf, new_uuid);
+		g_free (new_uuid);
+	}
+
 	if (street)
 		couchdb_document_contact_address_set_street (sf, street);
 	if (city)
@@ -424,16 +454,15 @@ couchdb_document_contact_address_get_street (CouchDBStructField *sf)
 {
 	g_return_val_if_fail (sf != NULL, NULL);
 
-	return json_object_get_string_member (sf->json_object, "address1");
+	return couchdb_struct_field_get_string_field (sf, "address1");
 }
 
 void
 couchdb_document_contact_address_set_street (CouchDBStructField *sf, const char *street)
 {
 	g_return_if_fail (sf != NULL);
-	g_return_if_fail (street != NULL);
 
-	json_object_set_string_member (sf->json_object, "address1", street);
+	couchdb_struct_field_set_string_field (sf, "address1", street);
 }
 
 const char *
@@ -441,16 +470,15 @@ couchdb_document_contact_address_get_city (CouchDBStructField *sf)
 {
 	g_return_val_if_fail (sf != NULL, NULL);
 
-	return json_object_get_string_member (sf->json_object, "city");
+	return couchdb_struct_field_get_string_field (sf, "city");
 }
 
 void
 couchdb_document_contact_address_set_city (CouchDBStructField *sf, const char *city)
 {
 	g_return_if_fail (sf != NULL);
-	g_return_if_fail (city != NULL);
 
-	json_object_set_string_member (sf->json_object, "city", city);
+	couchdb_struct_field_set_string_field (sf, "city", city);
 }
 
 const char *
@@ -458,16 +486,15 @@ couchdb_document_contact_address_get_state (CouchDBStructField *sf)
 {
 	g_return_val_if_fail (sf != NULL, NULL);
 
-	return json_object_get_string_member (sf->json_object, "state");
+	return couchdb_struct_field_get_string_field (sf, "state");
 }
 
 void
 couchdb_document_contact_address_set_state (CouchDBStructField *sf, const char *state)
 {
 	g_return_if_fail (sf != NULL);
-	g_return_if_fail (state != NULL);
 
-	json_object_set_string_member (sf->json_object, "state", state);
+	couchdb_struct_field_set_string_field (sf, "state", state);
 }
 
 const char *
@@ -475,16 +502,15 @@ couchdb_document_contact_address_get_country (CouchDBStructField *sf)
 {
 	g_return_val_if_fail (sf != NULL, NULL);
 
-	return json_object_get_string_member (sf->json_object, "country");
+	return couchdb_struct_field_get_string_field (sf, "country");
 }
 
 void
 couchdb_document_contact_address_set_country (CouchDBStructField *sf, const char *country)
 {
 	g_return_if_fail (sf != NULL);
-	g_return_if_fail (country != NULL);
 
-	json_object_set_string_member (sf->json_object, "country", country);
+	couchdb_struct_field_set_string_field (sf, "country", country);
 }
 
 const char *
@@ -492,16 +518,15 @@ couchdb_document_contact_address_get_postalcode (CouchDBStructField *sf)
 {
 	g_return_val_if_fail (sf != NULL, NULL);
 
-	return json_object_get_string_member (sf->json_object, "postalcode");
+	return couchdb_struct_field_get_string_field (sf, "postalcode");
 }
 
 void
 couchdb_document_contact_address_set_postalcode (CouchDBStructField *sf, const char *postalcode)
 {
 	g_return_if_fail (sf != NULL);
-	g_return_if_fail (postalcode != NULL);
 
-	json_object_set_string_member (sf->json_object, "postalcode", postalcode);
+	couchdb_struct_field_set_string_field (sf, "postalcode", postalcode);
 }
 
 const char *
@@ -509,16 +534,15 @@ couchdb_document_contact_address_get_pobox (CouchDBStructField *sf)
 {
 	g_return_val_if_fail (sf != NULL, NULL);
 
-	return json_object_get_string_member (sf->json_object, "pobox");
+	return couchdb_struct_field_get_string_field (sf, "pobox");
 }
 
 void
 couchdb_document_contact_address_set_pobox (CouchDBStructField *sf, const char *pobox)
 {
 	g_return_if_fail (sf != NULL);
-	g_return_if_fail (pobox != NULL);
 
-	json_object_set_string_member (sf->json_object, "pobox", pobox);
+	couchdb_struct_field_set_string_field (sf, "pobox", pobox);
 }
 
 const char *
@@ -526,14 +550,13 @@ couchdb_document_contact_address_get_description (CouchDBStructField *sf)
 {
 	g_return_val_if_fail (sf != NULL, NULL);
 
-	return json_object_get_string_member (sf->json_object, "description");
+	return couchdb_struct_field_get_string_field (sf, "description");
 }
 
 void
 couchdb_document_contact_address_set_description (CouchDBStructField *sf, const char *description)
 {
 	g_return_if_fail (sf != NULL);
-	g_return_if_fail (description != NULL);
 
-	json_object_set_string_member (sf->json_object, "description", description);
+	couchdb_struct_field_set_string_field (sf, "description", description);
 }
