@@ -63,7 +63,10 @@ couchdb_document_new (CouchDB *couchdb)
 
 	document = g_object_new (COUCHDB_TYPE_DOCUMENT, NULL);
 	document->couchdb = couchdb;
+	document->dbname = NULL;
+
 	document->root_node = json_node_new (JSON_NODE_OBJECT);
+	json_node_set_object (document->root_node, json_object_new ());
 
 	return document;
 }
@@ -82,7 +85,7 @@ couchdb_document_get (CouchDB *couchdb,
 	g_return_val_if_fail (dbname != NULL, NULL);
 	g_return_val_if_fail (docid != NULL, NULL);
 
-	url = g_strdup_printf ("http://%s/%s/%s", couchdb->hostname, dbname, docid);
+	url = g_strdup_printf ("%s/%s/%s", couchdb->hostname, dbname, docid);
 	parser = send_message_and_parse (couchdb, SOUP_METHOD_GET, url, NULL, error);
 	if (parser) {
 		document = g_object_new (COUCHDB_TYPE_DOCUMENT, NULL);
@@ -99,7 +102,9 @@ couchdb_document_get (CouchDB *couchdb,
 }
 
 gboolean
-couchdb_document_put (CouchDBDocument *document, GError **error)
+couchdb_document_put (CouchDBDocument *document,
+		      const char *dbname,
+		      GError **error)
 {
 	char *url, *body;
 	const char *id;
@@ -107,19 +112,15 @@ couchdb_document_put (CouchDBDocument *document, GError **error)
 	gboolean result = FALSE;
 
 	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), FALSE);
+	g_return_val_if_fail (dbname != NULL, FALSE);
 
-	if (!document->dbname) {
-		g_warning ("Document does not have a database associated to it");
-		return FALSE;
-	}
-
-	id = couchdb_document_get_string_field (document, "_id");
+	id = couchdb_document_get_id (document);
 	body = couchdb_document_to_string (document);
 	if (id) {
-		url = g_strdup_printf ("http://%s/%s/%s", document->couchdb->hostname, document->dbname, id);
+		url = g_strdup_printf ("%s/%s/%s", document->couchdb->hostname, dbname, id);
 		parser = send_message_and_parse (document->couchdb, SOUP_METHOD_PUT, url, body, error);
 	} else {
-		url = g_strdup_printf ("http://%s/%s/", document->couchdb->hostname, document->dbname);
+		url = g_strdup_printf ("%s/%s/", document->couchdb->hostname, dbname);
 		parser = send_message_and_parse (document->couchdb, SOUP_METHOD_POST, url, body, error);
 	}
 
@@ -127,10 +128,17 @@ couchdb_document_put (CouchDBDocument *document, GError **error)
 		JsonObject *object;
 
 		object = json_node_get_object (json_parser_get_root (parser));
-		couchdb_document_set_string_field (document, "_id", json_object_get_string_member (object, "id"));
-		couchdb_document_set_string_field (document, "_rev", json_object_get_string_member (object, "rev"));
+		couchdb_document_set_id (document, json_object_get_string_member (object, "id"));
+		couchdb_document_set_revision (document, json_object_get_string_member (object, "rev"));
 
 		g_object_unref (G_OBJECT (parser));
+
+		if (document->dbname) {
+			g_free (document->dbname);
+			document->dbname = g_strdup (dbname);
+		}
+
+		result = TRUE;
 	}
 
 	/* free memory */
@@ -140,6 +148,34 @@ couchdb_document_put (CouchDBDocument *document, GError **error)
 	return result;
 }
 
+gboolean
+couchdb_document_delete (CouchDBDocument *document, GError **error)
+{
+	const char *id, *revision;
+	char *url;
+	JsonParser *parser;
+	gboolean result = FALSE;
+
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), FALSE);
+
+	id = couchdb_document_get_id (document);
+	revision = couchdb_document_get_revision (document);
+	if (!id || !revision) /* we can't remove a document without an ID and/or a REVISION */
+		return FALSE;
+
+	url = g_strdup_printf ("%s/%s/%s?rev=%s", document->couchdb->hostname, document->dbname, id, revision);
+	parser = send_message_and_parse (document->couchdb, SOUP_METHOD_DELETE, url, NULL, error);
+	if (parser) {
+		g_object_unref (G_OBJECT (parser));
+
+		result = TRUE;
+	}
+
+	g_free (url);
+
+	return result;
+}
+
 const char *
 couchdb_document_get_id (CouchDBDocument *document)
 {
@@ -181,6 +217,17 @@ couchdb_document_get_revision (CouchDBDocument *document)
 	return NULL;
 }
 
+void
+couchdb_document_set_revision (CouchDBDocument *document, const char *revision)
+{
+	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
+	g_return_if_fail (revision != NULL);
+
+	json_object_set_string_member (json_node_get_object (document->root_node),
+				       "_rev",
+				       revision);
+}
+
 gboolean
 couchdb_document_is_contact (CouchDBDocument *document)
 {
@@ -188,7 +235,7 @@ couchdb_document_is_contact (CouchDBDocument *document)
 
 	return !g_ascii_strcasecmp (json_object_get_string_member (json_node_get_object (document->root_node),
 								   "record_type"),
-				    "http://example.com/contact-record");
+				    "http://www.freedesktop.org/wiki/Specifications/desktopcouch/contact");
 }
 
 gboolean
@@ -290,9 +337,14 @@ couchdb_document_set_string_field (CouchDBDocument *document, const char *field,
 	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
 	g_return_if_fail (field != NULL);
 
-	json_object_set_string_member (json_node_get_object (document->root_node),
-				       field,
-				       value);
+	if (value) {
+		json_object_set_string_member (json_node_get_object (document->root_node),
+					       field,
+					       value);
+	} else {
+		/* Remove field if it's a NULL value */
+		couchdb_document_remove_field (document, field);
+	}
 }
 
 CouchDBStructField *
