@@ -236,6 +236,7 @@ couchdb_struct_field_new (void)
 	sf = g_slice_new (CouchDBStructField);
 	sf->ref_count = 1;
 	sf->json_object = json_object_new ();
+	sf->uuid = NULL;
 
 	return sf;
 }
@@ -248,6 +249,7 @@ couchdb_struct_field_new_from_json_object (JsonObject *json_object)
 	sf = g_slice_new (CouchDBStructField);
 	sf->ref_count = 1;
 	sf->json_object = json_object_ref (json_object);
+	sf->uuid = NULL;
 
 	return sf;
 }
@@ -366,9 +368,13 @@ couchdb_struct_field_set_string_field (CouchDBStructField *sf, const char *field
 {
 	g_return_if_fail (sf != NULL);
 	g_return_if_fail (field != NULL);
-	g_return_if_fail (value != NULL);
 
-	json_object_set_string_member (sf->json_object, field, value);
+	if (value)
+		json_object_set_string_member (sf->json_object, field, value);
+	else {
+		/* Remove the field if the value is NULL */
+		couchdb_struct_field_remove_field (sf, field);
+	}
 }
 
 CouchDBStructField *
@@ -390,3 +396,22 @@ couchdb_struct_field_set_struct_field (CouchDBStructField *sf, const char *field
 
 	json_object_set_object_member (sf->json_object, field, json_object_ref (value->json_object));
 }
+
+const char *
+couchdb_struct_field_get_uuid (CouchDBStructField *sf)
+{
+	g_return_val_if_fail (sf != NULL, NULL);
+
+	return (const char *) sf->uuid;
+}
+
+void
+couchdb_struct_field_set_uuid (CouchDBStructField *sf, const char *uuid)
+{
+	g_return_if_fail (sf != NULL);
+
+	if (sf->uuid)
+		g_free (sf->uuid);
+
+	sf->uuid = g_strdup (uuid);
+}
