@@ -19,6 +19,8 @@
  * Boston, MA 02110-1301, USA.
  */
 
+#include <uuid/uuid.h>
+#include <string.h>
 #include <libsoup/soup-session-async.h>
 #include "couchdb-glib.h"
 #include "utils.h"
@@ -85,3 +87,16 @@ send_message_and_parse (CouchDB *couchdb, const char *method, const char *url, c
 
 	return parser;
 }
+
+char *
+generate_uuid (void)
+{
+	uuid_t uuid;
+	char uuid_string[37];
+
+	uuid_generate_random (uuid);
+	uuid_unparse (uuid, uuid_string);
+
+	g_print ("Generated %s uuid", uuid_string);
+	return g_strdup (uuid_string);
+}
