@@ -69,8 +69,10 @@ CouchDBDocumentInfo *couchdb_document_info_new (const char *docid, const char *r
 
 struct _CouchDBStructField {
 	gint ref_count;
-
 	JsonObject *json_object;
+
+	/* Extra data needed for some specific StructField's */
+	char *uuid; /* the UUID of this item */
 };
 
 CouchDBStructField *couchdb_struct_field_new_from_json_object (JsonObject *json_object);
@@ -85,4 +87,6 @@ JsonParser *send_message_and_parse (CouchDB *couchdb,
 				    const char *body,
 				    GError **error);
 
+char *generate_uuid (void);
+
 #endif
